@@ -39,6 +39,9 @@
 #include "deskypplet.h"
 #include <hildon-mime.h>
 #include <libosso.h>
+#include <sys/statvfs.h>
+#include <glib.h>
+
 
 YNCtx *app=NULL;
 
@@ -373,43 +376,70 @@ void on_orientation_changed(GdkScreen *screen, YNCtx *app)
 
 void folder_down (GObject *o,GdkEventButton *event,gchar *fullpath)
 {   
-	/*const char *name;
-    char *basename;
-    basename = strrchr(fullpath,'/') + 1;
-    gchar *name=g_strconcat ("./",basename,"\0",NULL);*/
-	
-	int retval;
+	char var [100],value[100],line[200];
+	gchar *test;
+	char *mime_type [100];
+	char *mime_com [100];
+	int retval,i;
 	DBusConnection *dbus = NULL;
 	osso_context_t *osso;
 	stat(fullpath, &buf);
 	gchar *file_open_txpad=g_strconcat ("txpad ",fullpath,"\0",NULL);
-	gchar *file_open_deb=g_strconcat ("sudo dpkg -i ",fullpath,"\0",NULL);
+	gchar *file_open;
 	
 	if(S_ISDIR(buf.st_mode))
       {
 	   app->work_dir=fullpath;
        gtk_widget_destroy(app->table);
-	   create_table (app);
+	   create_table (app);return;
 	   }	
-      else
-       {if(strstr(fullpath,".deb"))
-        {system (file_open_deb);}
-	    else
-	    {
-	    osso = osso_initialize("osso-mime-summon", "0.1", TRUE, NULL);
-        if (osso == NULL) 
-            {printf("Failed to initialize libosso\n");}
-        dbus = (DBusConnection *) osso_get_dbus_connection(osso);
-        if (dbus == NULL) 
-           {printf("Failed to get dbus connection from osso context\n");}
-        retval = hildon_mime_open_file(dbus, fullpath);
-	    if (retval != 1) 
-	       {printf("Failed to launch hildon_mime_open_file: %d\n", retval);
-	       system (file_open_txpad);}
-        }
-	    }
-	
-    return;
+    else
+      {
+	   FILE *fp=fopen("/home/user/.deskypplet/mimehandling.conf","r");
+       i=0;
+	if (fp) 
+      {
+	   while (fgets(line,sizeof(line),fp))
+          {
+	       i++;
+           memset (var,0,sizeof(var));
+           memset (value,0,sizeof(value));
+		   sscanf(line,"%[^'=']=%[^\'#'\n]%*[\t]%*[\t]",var,value);  
+		   mime_type [i]=g_strconcat (var,NULL);
+		   mime_com [i]=g_strconcat (value,"\0",NULL);
+		   test=g_strconcat (mime_type [i],NULL);
+		   if(strstr(fullpath,mime_type[i]))
+             {
+			  file_open=g_strconcat (mime_com[i]," ",fullpath,"\0",NULL);
+			  system (file_open);
+			  return;
+			  }
+		   /*printf("%s",file_open)
+		   printf ("test %s\n",test);
+		   printf ("fullpath %s\n",fullpath);
+		   printf ("mime %s\n",var);
+		   printf ("com%s\n\n",value);*/
+		  }
+	   fclose (fp);
+	   }
+	  		
+		}
+	   
+	   osso = osso_initialize("osso-mime-summon", "0.1", TRUE, NULL);
+       if (osso == NULL) 
+       {printf("Failed to initialize libosso\n");}
+       dbus = (DBusConnection *) osso_get_dbus_connection(osso);
+       if (dbus == NULL) 
+       {printf("Failed to get dbus connection from osso context\n");}
+       retval = hildon_mime_open_file(dbus, fullpath);
+	   if (retval != 1) 
+	      {
+	       printf("Failed to launch hildon_mime_open_file: %d\n", retval);
+	       system (file_open_txpad);
+	       }
+         
+	  
+	return;
 }
 
 void go_folder_up (GObject *o,GdkEventButton *event,YNCtx *app)
@@ -557,43 +587,28 @@ void create_table (YNCtx *app)
        		    {filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/folder.png");}
              }
          }
-       else if(strstr(name[a],".zip") ||strstr(name[a],"tar") 
-                       ||strstr(name[a],"gz"))
-       {
-		 filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/packed.png");
-        } 
+       else if(strstr(name[a],".zip") ||strstr(name[a],".tar") ||strstr(name[a],".gz"))
+             {filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/packed.png");} 
        else if(strstr(name[a],".txt"))
-       {
-		 filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/txt.png");
-        }
+             {filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/txt.png");}
        else if(strstr(name[a],".pdf"))
-       {
-		 filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/pdf.png");
-        }
+             {filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/pdf.png");}
        else if(strstr(name[a],".py"))
-       {
-		 filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/py.png");
-        }
-        else if(strstr(name[a],".sh"))
-       {
-		 filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/sh.png");
-        }
-        else if(strstr(name[a],".deb"))
-       {
-		 filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/deb.png");
-        }
+             {filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/py.png");}
+       else if(strstr(name[a],".sh"))
+             {filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/sh.png");}
+       else if(strstr(name[a],".deb"))
+             {filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/deb.png");}
        else if(strstr(name[a],".mp3") ||strstr(name[a],".ogg"))
-       {
-		 filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/mp3.png");
-        }
+             {filebrowser_fb_img=gtk_image_new_from_file("/home/user/.deskypplet/icons/mp3.png");}
        else
-        {
-	     if ((pixbuf= gdk_pixbuf_new_from_file_at_size (fullpath,80,80,NULL))==NULL)   
-		   {
-			pixbuf=gdk_pixbuf_new_from_file_at_size ("/home/user/.deskypplet/icons/dummy.png",80,80,NULL);
-            }
-		 filebrowser_fb_img=gtk_image_new_from_pixbuf(pixbuf);
-	    }
+            {
+	         if ((pixbuf= gdk_pixbuf_new_from_file_at_size (fullpath,80,80,NULL))==NULL)   
+		       {
+			    pixbuf=gdk_pixbuf_new_from_file_at_size ("/home/user/.deskypplet/icons/dummy.png",80,80,NULL);
+               }
+		     filebrowser_fb_img=gtk_image_new_from_pixbuf(pixbuf);
+	         }
       
        gtk_container_add (GTK_CONTAINER (filebrowser_fb_evbox),filebrowser_fb_img);
 	   	
@@ -613,10 +628,7 @@ void create_table (YNCtx *app)
 	 
 	   gtk_table_attach_defaults (GTK_TABLE (app->table), filebrowser_fb_evbox,i,i+1,j,j+1);
        gtk_table_attach_defaults (GTK_TABLE (app->table), label,i,i+1,j+1,j+2);
-     /* g_signal_connect (app->folder_up_evbox, "expose-event", G_CALLBACK (handle_expose),
-                           app->folder_up_evbox);
-    */
-      }
+    }
      
      app->folder_up_image=gtk_image_new_from_file("./icons/folder_up.png");
 	app->folder_up_evbox=gtk_event_box_new ();
@@ -654,26 +666,33 @@ void menu ()
 {
 	gchar *com_do=NULL;
 	gchar *fullpath=NULL;
-	char var [50],value[100],line[150];
+	char var [100],value[100],line[200];
 	int i=0;
 	GtkWidget *vbox=NULL;
 	GtkWidget *hbox=NULL;
 	GtkWidget *button=NULL;
 	GtkWidget *button_term=NULL;
+	GtkWidget *button_fav=NULL;
 	GtkWidget *pannable_area=NULL;
     GtkWidget *table=NULL;
 	GtkWidget *label=NULL;
 	
+	gchar *title;
+	gchar *title_capa;
+	gchar *title_free;
+	title_capa=g_get_capacity (app->work_dir);
+	title_free=g_get_free_space (app->work_dir);
+	title=g_strconcat ("free ",title_free," / ",title_capa,"\0",NULL);
 	
-	dialog=gtk_dialog_new_with_buttons("Menu",NULL, 0,0,0,0,0,NULL);
+	dialog=gtk_dialog_new_with_buttons(title,NULL, 0,0,0,0,0,NULL);
 	gtk_widget_set_size_request (GTK_WIDGET(dialog),-1,800); 
     
     hbox=gtk_hbox_new(TRUE,0);
 	vbox=gtk_vbox_new(TRUE,0);
 	
 	pannable_area = hildon_pannable_area_new ();
-	gtk_widget_set_size_request (GTK_WIDGET(pannable_area),800,-1); 
-    
+	/*gtk_widget_set_size_request (GTK_WIDGET(pannable_area),app->applet_x_size-20,-1); 
+    */
 	table = gtk_table_new (40,2, FALSE);
     gtk_table_set_row_spacings (GTK_TABLE (table), 2);
     gtk_table_set_col_spacings (GTK_TABLE (table), 2);
@@ -683,15 +702,22 @@ void menu ()
       {
 	   while (fgets(line,sizeof(line),fp))
           {
-	       sscanf(line, "%[^\n]\n", value);
-	       /* printf ("%s\n",line);printf ("%s\n",value);*/
+	       /*sscanf(line, "%[^\n]\n", value);
+	        printf ("line %s\n",line);printf ("%s\n",value);*/
 	       i++;
+          memset (var,0,sizeof(var));
+           memset (value,0,sizeof(value));
+		    sscanf(line,"%[^'=']=%[^\'#'\n]%*[\t]%*[\t]",var,value);  
+			
            button = hildon_button_new_with_text (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT,
 		    				       HILDON_BUTTON_ARRANGEMENT_HORIZONTAL, 0,0);
 		   fullpath=g_strconcat (value,"\0",NULL);
-	       label=gtk_label_new (fullpath);
-	       /*gtk_widget_set_size_request (GTK_WIDGET(label),350,-1);*/ 
-    
+		   if (fullpath==NULL){fullpath="/home/user";}
+	       label=gtk_label_new (var);
+	       printf ("\n%s\n",var);
+			  printf ("%s\n",value);
+	       /*gtk_widget_set_size_request (GTK_WIDGET(label),app->applet_x_size-30,-1); 
+    */
 	       gtk_label_set_line_wrap (GTK_LABEL (label),TRUE);
            gtk_label_set_line_wrap_mode (GTK_LABEL (label), PANGO_WRAP_WORD_CHAR);
            gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.5f);
@@ -703,16 +729,15 @@ void menu ()
            }
 	   fclose (fp);
 	   }
-	   
-	i=1;
+	i=2;
 	fp=fopen("/home/user/.deskypplet/yppletactions.conf","r");
 	if (fp) 
       {
 	   while (fgets(line,sizeof(line),fp))
           {
 	       i++;
-	       memset (var,0,sizeof(var));
-           memset (value,0,sizeof(value));
+	      /* memset (var,0,sizeof(var));
+           memset (value,0,sizeof(value));*/
 		    sscanf(line,"%[^'=']=%[^\'#'\n]%*[\t]%*[\t]",var,value);  
 			 /* printf ("\n%s\n",var);
 			  printf ("%s\n",value);
@@ -734,7 +759,13 @@ void menu ()
 	                				       HILDON_BUTTON_ARRANGEMENT_VERTICAL,"Terminal here",NULL);
 	g_signal_connect (button_term, "clicked", G_CALLBACK (term_her),NULL);
     gtk_button_set_alignment (GTK_BUTTON(button_term),0,0.5);				       
-	gtk_table_attach_defaults (GTK_TABLE (table), button_term,1,2,1,2);     
+	gtk_table_attach_defaults (GTK_TABLE (table), button_term,1,2,1,2);  
+	
+	button_fav = hildon_button_new_with_text (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT,
+	                				       HILDON_BUTTON_ARRANGEMENT_VERTICAL,"Bookmark folder",NULL);
+	g_signal_connect (button_fav, "clicked", G_CALLBACK (book_this),NULL);
+    gtk_button_set_alignment (GTK_BUTTON(button_fav),0,0.5);				       
+	gtk_table_attach_defaults (GTK_TABLE (table), button_fav,1,2,2,3);     
 	
 	hildon_pannable_area_add_with_viewport (HILDON_PANNABLE_AREA (pannable_area),table);
     gtk_container_add (GTK_CONTAINER (hbox), pannable_area);
@@ -761,13 +792,9 @@ void bookmark (gpointer button,gchar *fullpath)
 void custom_com (gpointer button,gchar *com_do)
 {   
 	com_com=g_strconcat (com_do,app->work_dir,"\" & ","\0",NULL);
-	
-	/*com_com=g_strconcat (hildon_button_get_value(button),app->work_dir,"\" & ","\0",NULL);
-	*/system (com_com);
+	system (com_com);
 	/*printf ("com_com %s",com_com);*/
     gtk_widget_destroy(dialog);
-	/*gtk_widget_destroy(app->table);
-	create_table (app);*/
 	return;
 }
 
@@ -787,8 +814,44 @@ void term_her ()
 	
 	system ("osso-xterm /home/user/.deskypplet/deskypplet-terminal &");
 	gtk_widget_destroy(dialog);
-	/*gtk_widget_destroy(app->table);
-	create_table (app);*/
+	return;
+}
+
+void book_this ()
+{   gchar *fullpath;
+	char line[150],value[150];
+	char *bookm[150];
+	int i=0,j=0;
+	FILE *fp=fopen("/home/user/.deskypplet/ypplet-bookmarks.conf","r");
+	if (fp) 
+      {
+	   while (fgets(line,sizeof(line),fp))
+          {
+	       /*printf ("xxline %s\n",line);*/
+	       sscanf(line, "%[^\n]\n", value);
+	       bookm[i]=g_strconcat (value,"\0",NULL);
+	       i++;
+           }
+	   fclose (fp);
+	   }
+	   fullpath=g_strconcat (app->work_dir,"=",app->work_dir,"\0",NULL);
+	
+	   /*printf ("oooxline %s\n",fullpath);
+	   printf ("1line %s\n",bookm [1]);
+	   hildon_banner_show_information(NULL, NULL, "Bookmark added to list");*/
+	 
+	fp=fopen("/home/user/.deskypplet/ypplet-bookmarks.conf","w");
+	if(fp!=NULL)
+	  {
+		for (j = 0; j < i; j++) 
+        {
+		fprintf(fp,"%s\n",bookm[j]);
+		}
+		fprintf(fp,"%s\n",fullpath);
+		fclose(fp);
+		hildon_banner_show_information(NULL, NULL, "Bookmark added to list");   
+	   }
+	gtk_widget_destroy(dialog);
 	return;
 }
 
@@ -843,13 +906,58 @@ void do_it (void)
 	
 	return;*/
 	}
+
+
+
+
+gchar *g_get_capacity ( gchar * dev_path)
+{
+	unsigned long long result = 0;
+	int n;
+	gchar s_cap[50];
+	gchar * ss_cap = "N/A";
+	struct statvfs sfs;
+	if ( statvfs ( dev_path, &sfs) != -1 )
+	{
+		result = (unsigned long long)sfs.f_bsize * sfs.f_blocks;
+	}
+	if (result > 0)
+	{
+		double f_cap = (double)result/(1024*1024*1024);
+		n = sprintf(s_cap, "%.3f Gb", f_cap);
+		ss_cap = g_strdup(s_cap);
+	}
+	return ss_cap;
+}
+
+gchar *g_get_free_space ( gchar * dev_path)
+{
+	unsigned long long result = 0;
+	int n;
+	gchar s_cap[50];
+	gchar * ss_cap = "N/A";
+	struct statvfs sfs;
+	if ( statvfs ( dev_path, &sfs) != -1 )
+	{
+		result = (unsigned long long)sfs.f_bsize * sfs.f_bfree;
+	}
+	if (result > 0)
+	{
+		double f_cap = (double)result/(1024*1024*1024);
+		n = sprintf(s_cap, "%.3f Gb", f_cap);
+		ss_cap = g_strdup(s_cap);
+	}
+	return ss_cap;
+}
+
 /*new:
+ add bookmark button
+ bookmarks file format changed 
+ Size in partition left in menu title
  
 to do:
-size left
+portrait adjust not fullscreen
 execute edit sh
 unset pixbuf
-ask button title/value or title
 .TXT .txt icons bug
- */
-
+*/
